"""
Dummy object factory doing nothing, for testing the interpreter
$Id: DummyFactory.py,v 1.3 2000/08/29 10:51:42 konrad Exp $
(C) 2000 Konrad Anton, Cedric Adjih
"""
#---------------------------------------------------------------------------
# taken from:
# Cedric Adjih - 2000 - adjih@crepuscule.com - ICFP'200 Programming Contest
#---------------------------------------------------------------------------
import GMLTypes, Exceptions, Ops

class AbstractSceneFactory:
    def isSceneObject(self, object):                       undefined
    def isLightObject(self, object):                       undefined
    
    def makeSphere(self, surfaceFunction):                 undefined
    def makeCube(self, surfaceFunction):                   undefined 
    def makeCylinder(self, surfaceFunction):               undefined 
    def makeCone(self, surfaceFunction):                   undefined 
    def makePlane(self, surfaceFunction):                  undefined
    
    def applyTranslation(self, sceneObject, rx, ry, rz):   undefined
    def applyScale(self, sceneObject, rx, ry, rz):         undefined
    def applyUScale(self, sceneObject, s):                 undefined
    def applyRotateX(self, sceneObject, angle):            undefined
    def applyRotateY(self, sceneObject, angle):            undefined
    def applyRotateZ(self, sceneObject, angle):            undefined

    def makeLight(self, direction, color):                 undefined
    def makePointLight(self, direction, color):            undefined
    def makeSpotLight(self, pos, dir, color, cutoff, exp): undefined
    
    def makeUnion(self, object1, object2):                 undefined
    def makeIntersect(self, object1, object2):             undefined
    def makeDifference(self, object1, object2):            undefined

#---------------------------------------------------------------------------

GeomPrimitive=["sphere","cube","cylinder","cone","plane"]

class SimpleSceneObject:
    def __init__(self,nudgeclo,*data):
        self.nudgefun=nudgeclo
        self.data=data
    def __getitem__(self, index):
        return self.data[index]
    def __len__(self):
        return len(self.data)
    def nudge(self, greeting):
        """
        nudging is defined as calling a scene object's closure with
        a string as greeting parameter.
        """
        print "(NUDGE", self.data[0],")"
        stdata=[greeting]
        self.nudgefun(stdata)
        
    def getSceneToRender(self):
        currentTransform=Transform()
        return self.getSceneWithTransform(currentTransform)
    def getSceneWithTransform(self, transform):
        if self[0]=="transform":
            return self[2].getSceneWithTransform(transform*self[1])
        elif self[0] in ["union", "intersect", "difference"]:
            return [self[0], self[1].getSceneWithTransform(transform),
                    self[2].getSceneWithTransform(transform)]
        else:
            assert self[0] in GeomPrimitive
            return [self[0], transform]

class SimpleLightObject:
    def __init__(self, *data):
        self.data=data
        self.kind=data[0]
    def __getitem__(self, index):
        return self.data[index]
    def nudge(self):
        print "Blink-blink goes the", self.kind

class SimpleSceneFactory:
    """Note: each sceneObject must have checks first"""
    def isSceneObject(self, object):
        return isinstance(object, SimpleSceneObject)
    def isLight(self, object):
        return isinstance(object, SimpleLightObject)

    #--- Geometric primitives

    def makeSphere(self, surfaceFunction):
        return SimpleSceneObject(surfaceFunction, "sphere")
    def makeCube(self, surfaceFunction):
        return SimpleSceneObject(surfaceFunction, "cube")
    def makeCylinder(self, surfaceFunction): 
        return SimpleSceneObject(surfaceFunction, "cylinder")
    def makeCone(self, surfaceFunction):
        return SimpleSceneObject(surfaceFunction, "cone")
    def makePlane(self, surfaceFunction):
        return SimpleSceneObject(surfaceFunction, "plane")

    #--- Transforms
    
    def applyTranslation(self, sceneObject, rx, ry, rz):
        return SimpleSceneObject(sceneObject.nudgefun,
                                 "translate %f %f %f"%(rx,ry,rz),
                                 sceneObject)
    
    def applyScale(self, sceneObject, rx, ry, rz):
        return SimpleSceneObject(sceneObject.nudgefun,
                                 "scale %f %f %f"%(rx,ry,rz),
                                 sceneObject)
    
    def applyUScale(self, sceneObject, scale):
        return SimpleSceneObject(sceneObject.nudgefun,
                                 "uscale %f"%scale, sceneObject)
        
    def applyRotateX(self, sceneObject, angle):
        return SimpleSceneObject(sceneObject.nudgefun,"rotatex %f"%angle,
                                 sceneObject)
    def applyRotateY(self, sceneObject, angle):
        return SimpleSceneObject(sceneObject.nudgefun,"rotatey %f"%angle,
                                 sceneObject)
    def applyRotateZ(self, sceneObject, angle):
        return SimpleSceneObject(sceneObject.nudgefun,"rotatez %f"%angle,
                                 sceneObject)

    #--- Lights
    
    def makeLight(self, direction, color):
        return SimpleLightObject("light %s %s"%(str(direction),str(color)),
                                 direction, color)
    def makePointLight(self, direction, color):
        return SimpleLightObject("pointlight %s %s"%(str(direction),
                                                     str(color)),
                                 direction, color)
    def makeSpotLight(self, pos, dir, color, cutoff, exp):
        return SimpleLightObject("spotlight %s %s %s %f %f"%\
                                 (str(pos), str(dir), str(color), cutoff, exp))
    
    def makeUnion(self, object1, object2):
        return SimpleSceneObject(object1.nudgefun, "union", object1, object2)
    def makeIntersect(self, object1, object2):
        return SimpleSceneObject(object1.nudgefun,
                                 "intersect", object1, object2)
    def makeDifference(self, object1, object2):
        return SimpleSceneObject(object1.nudgefun,
                                 "difference", object1, object2)

#---------------------------------------------------------------------------
