#---------------------------------------------------------------------------
# Cobra Team - ICFP'200 Programming Contest
#
# Cedric Adjih - 2000 - adjih@crepuscule.com 
#---------------------------------------------------------------------------

import math

MatrixRepr=0

# Quasi-infinity
Infinity=1e20
MinusInfinity=-Infinity

Epsilon=1e-6
Omega=1e6
OmegaSquare=Omega*Omega
def isZero(x):
    return -Epsilon<=x<=Epsilon
def isInfinity(x):
    return x>Omega or x<Omega

def safeInverse(x):
    try: result=1.0/x
    except ZeroDivisionError:
        if x>0: return Infinity
        else: return -Infinity
    return result


def degreeToRadian(x):
    return x/180.0*math.pi

def radianToDegree(x):
    return x/math.pi*180.0

#---------------------------------------------------------------------------

# Using convention (matrix M multiplied by vector is: M V
# not V M).
#
# [ m11 m12 m13 tx ] -------/ 
# [ m21 m22 m23 ty ] /------> stored this way
# [ m31 m32 m33 tz ]
# [  0   0   0   1 ]

IdentityData=[1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1]
ZeroData=[0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0]

class Transform:
    """A transform implemented as a 4x4 matrix
    it also holds and compute the inverse matrix"""

    def __init__(self,):
        self.toIdentity()
        self.inverseOk=1

    def _notforuse_fix(self):
        self.data[12:16]=[0,0,0,1]

    def dup(self):
        result=Transform()
        result.data=self.data[:]
        result.inverseData=self.data[:]
        return result

    def asInverse(self):
        result=Transform()
        result.data=self.inverseData[:]
        result.inverseData=self.data[:]
        return result

#     def asTranspose(self):
#         result=Transform()
#         for i in range(4):
#             for j in range(4):
#                 result.set(i,j,self.get(j,i))
#                 result.setInverse(i,j,self.getInverse(j,i))
#         return result

    def toScale(self, diagonal):
        assert len(diagonal)==3
        self.data=[0]*4*4
        self.inverseData=[0]*4*4
        self.set(3,3, 1.0)
        self.setInverse(3,3, 1.0)
        for i in range(3):
            self.set(i,i, diagonal[i])
            self.setInverse(i,i, safeInverse(diagonal[i]))
            
    def toIdentity(self):
        self.data=IdentityData[:]
        self.inverseData=IdentityData[:]

    def toZero(self):
        self.data=ZeroData[:]
        self.inverseData=ZeroData[:]
        self.inverseOk=0
        
    def toTranslation(self, translation):
        self.data=IdentityData[:]
        for i in range(3):
            self.set(i,3, translation[i])
            self.setInverse(i,3, -translation[i])

    def toRotationX(self, angle):
        """angle is in degrees"""
        self.toIdentity()
        angle=degreeToRadian(angle)
        c=math.cos(angle)
        s=math.sin(angle)
        self.set(1,1, c)  ; self.setInverse(1,1, c)
        self.set(1,2, -s) ; self.setInverse(1,2, s)
        self.set(2,1, s)  ; self.setInverse(2,1, -s)
        self.set(2,2, c)  ; self.setInverse(2,2, c)

    def toRotationY(self, angle):
        """angle is in degrees"""
        self.toIdentity()
        angle=degreeToRadian(angle)
        c=math.cos(angle)
        s=math.sin(angle)
        self.set(0,0, c)  ; self.setInverse(0,0, c)
        self.set(0,2, s)  ; self.setInverse(0,2, -s)
        self.set(2,0, -s) ; self.setInverse(2,0, s)
        self.set(2,2, c)  ; self.setInverse(2,2, c)

    def toRotationZ(self, angle):
        """angle is in degrees"""
        self.toIdentity()
        angle=degreeToRadian(angle)
        c=math.cos(angle)
        s=math.sin(angle)
        self.set(0,0, c)  ; self.setInverse(0,0, c)
        self.set(0,1, -s) ; self.setInverse(0,1, s)
        self.set(1,0, s)  ; self.setInverse(1,0, -s)
        self.set(1,1, c)  ; self.setInverse(1,1, c)  

    def get(self,line,column): # not (x,y) but (y,x)
        return self.data[line*4+column]

    def getInverse(self,line,column):
        return self.inverseData[line*4+column]
    
    def set(self, line, column, value): # not (x,y) but (y,x)
        self.data[line*4+column]=value
        
    def setInverse(self, line, column, value):
        self.inverseData[line*4+column]=value
        
    def mul(self, vector, lastComponent=1.0):
        assert len(vector)==3
        vector=vector+[lastComponent]
        result=[0.0]*4
        for line in range(4):
            for column in range(4):
                result[line]=result[line]+self.get(line,column)*vector[column]
        return result[0:3]

    def inverseMul(self, vector, lastComponent=1.0):
        assert len(vector)==3
        vector=vector+[lastComponent]
        result=[0.0]*4
        for line in range(4):
            for column in range(4):
                result[line]=result[line]\
                              +self.getInverse(line,column)*vector[column]
        return result[0:3]
        
    def __mul__(self, other):
        result=Transform()
        result.toZero()
        for i in range(0,4):
            for j in range(0,4):
                for k in range(0,4):
                    result.set(i,j, result.get(i,j)\
                               +self.get(i,k)*other.get(k,j))
                    result.setInverse(i,j, result.getInverse(i,j)\
                              +other.getInverse(i,k)*self.getInverse(k,j))
        result.inverseOk=self.inverseOk and other.inverseOk
        return result

    def __repr__(self):
        if MatrixRepr>0:
            result=("""<Matrix %f %f %f %f
        %f %f %f %f
        %f %f %f %f
        %f %f %f %f\n""" % tuple(self.data))
        else: result="<Matrix>"
        if MatrixRepr>1:
            result=result+"""<Matrix %f %f %f %f
        %f %f %f %f
        %f %f %f %f
        %f %f %f %f\n""" % tuple(self.inverseData)
        return result

#---------------------------------------------------------------------------
# Some Vectorial mathematics

import operator

def distance(vertex1, vertex2):
    return math.sqrt(reduce(operator.add,
                            map(lambda x,y: (x-y)*(x-y), vertex1, vertex2)))

def findMostFar(referenceList, vertexList):
    maxDistance=-1.0  # distances are >= 0
    index="<no result, in findMostFar>"
    for i in range(len(vertexList)):
        distanceList=map(distance, [vertexList[i]]*len(referenceList),
                         referenceList)
        currentDistance=min(distanceList)
        if currentDistance>=maxDistance:
            maxDistance=currentDistance
            index=i
    return index, maxDistance

def mulVector(v1,v2): return map(operator.mul, v1,v2)
def subVector(v1,v2): return map(operator.sub, v1,v2)
def addVector(v1,v2): return map(operator.add, v1,v2)
def scaleVector(scale,v): return map(lambda x,s=scale: s*x, v)
def dotProdVector(v1,v2): return reduce(operator.add,map(operator.mul,v1,v2))
def normVector(v): return distance(v,[0,0,0])
def normalizeVector(v): return scaleVector(safeInverse(normVector(v)), v)
def crossProdVector(u,v):
    return [u[1]*v[2]-u[2]*v[1], u[2]*v[0]-u[0]*v[2], u[0]*v[1]-u[1]*v[0]]

def ray(v1, t, v2): return addVector(v1, scaleVector(t, v2))

#---------------------------------------------------------------------------
# Vector class: it was added when I realized how much easier it
# was to use common operators.

def toVect(data):
    if type(data)==type([]):
        return Vector(data)
    else: return data

class Vector:
    def __init__(self, data):
        assert len(data)==3
        self.data=data[:]
    def __rmul__(self, other):
        other=toVect(other)
        if type(other)==type(1.0) or type(other)==type(1):
            return Vector(scaleVector(other, self.data))
        #XXX: elseXXX
    def __and__(self, other):
        other=toVect(other)
        return dotProdVector(self.data, other.data)
    def __add__(self,other):
        other=toVect(other)
        return Vector(addVector(self.data, other.data))
    def __sub__(self,other): #XXX: check
        other=toVect(other)
        return Vector(subVector(self.data, other.data))
    def __repr__(self):
        return "<"+repr(self.data)+">"
    def __getitem__(self,index):
        return self.data[index]
    def normalize(self):
        return Vector(normalizeVector(self.data))
    def safeNormalize(self, fallBack):
        try: return Vector(scaleVector(1.0/normVector(self.data), self.data))
        except: return fallBack
    def __len__(self): return len(self.data)
    def __or__(self,other):
        other=toVect(other)
        return Vector(mulVector(self.data,other.data))



#---------------------------------------------------------------------------

def testTransform():
    global MatrixRepr
    MatrixRepr=2
    import whrandom
    rand=whrandom.random
    t=Transform()
    s=math.exp(0.95)
    sequence=map(lambda x,rand=rand: int(rand()*5), range(100))
    for v in sequence:
        newTransform=Transform()
        if v==0: newTransform.toTranslation([rand()-0.5, rand()-0.5,
                                             rand()-0.5])
        elif v==1: newTransform.toRotationX(rand()*360)
        elif v==2: newTransform.toRotationY(rand()*360)
        elif v==3: newTransform.toRotationZ(rand()*360)
        elif v==4: newTransform.toScale([s*rand(), s*rand(), s*rand()])
        t=t*newTransform
    print "Transform:\n",t
    t2=t.asInverse()
    print "InverseTransform*Transform (should be identity):\n",t2*t

#---------------------------------------------------------------------------
        
if __name__=="__main__":
    print Vector([0,0.1,0]).safeNormalize([1,1,1])
    print Vector([0,0,0]).safeNormalize([1,1,1])
    print ("degree/radian: should be 123.0",
           degreeToRadian(radianToDegree(123.0)))
    testTransform()

#---------------------------------------------------------------------------
